;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Mike Byrne & Frank Tamborello
;;; Copyright   : (c)2007 Rice U./Mike Byrne, All Rights Reserved
;;; Availability: public domain
;;; Address     : Rice University
;;;             : Psychology Department
;;;             : Houston,TX 77251-1892
;;;             : byrne@acm.org
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : X84-model.lisp
;;; Version     : r5
;;; 
;;; Description : Code to run X84, Frank Trek 11: Semantic Space, with ACT-R.
;;; 
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;; 2008.11.14 fpt 
;;;		: do-stimgnen can now generate a block of a single trial kind, since 
;;;		procedure-window2's finish-task method now calls (finish-trial *experiment*)
;;;		to handle data collection, now that this thing no longer uses modal dialogs.
;;; 2008.10.14 fpt [r5]
;;;		: Adds a dummy trial during stimgen so that ACT-R can complete all of
;;;		experiment trials. After the modal-dialog had been eliminated, 
;;;		main-control-window was calling finish-trial before the procedure-window
;;;		task had finished, and thus finish-block and finish-experiment were
;;;		called before the last trial could be done.
;;; 2008.08.28 fpt [r4]
;;;		: Commented out portion of INITIALIZE-INSTANCE (main-control-window)
;;;		that shows instructions at start of day 2.
;;;		: Commented out portion of DO-BUTTON (MAIN-CONTROL-WINDOW) that
;;;		shows instructions half-way through the experiment. I may want to
;;;		co-opt that portion of DO-BUTTON to make the declarative chunks appear
;;;		that tell ACT-R how to do the new task sequence.
;;;		: Revised SETUP-TRIAL (MAIN-CONTROL-WINDOW) to just go ahead and call
;;;		DO-BUTTON (MAIN-CONTROL-WINDOW) with the CURRENT-BTN of main-control-window
;;;		so that the behind the scenes functions of MAIN-CONTROL-WINDOW are all
;;;		preserved, but there's now no Main Control task.
;;;		: Added method INSTRUCT-ACTR (main-control-window) so that ACT-R can get
;;;		its experiment change instructions in the form of new DM chunks.
;;; 2008.08.26 fpt [r3]
;;;		: For the sake of speeding up model runs, I altered... 
;;;		- transporter-window-switchable so that it starts with only two
;;;		scanner dots, the dots move faster, and CEASE-TRACKING and 
;;;		GIVE-FEEDBACK no longer play sound effects.
;;;		- window-null-event-handler (phaser-window) to just set the 
;;;		thermometer value to 60.
;;; 2008.08.26 fpt [r2]
;;;		: Give ACT-R ears!
;;;		: Overrode wm-tester's NEW-PROBE method so that if *ACTR-ENBALED-P*,
;;;		it generates the appropriate NEW-WORD-SOUND event, or else it
;;;		plays the sound as before.
;;;		: Overrode wm-tester's DO-PROBE method so that ... NEW-TONE-SOUND,
;;;		or else it plays the sound as before.
;;; 2008.08.25 fpt [r1]
;;;		: Altered transporter-window-switchable so that the target
;;;		no longer moves.
;;;		: Renamed file to reflect fact that it's been altered from
;;;		the behavioral studies version for use with the ACT-R model.
;;;
;;; 2008.03.20 fpt [r3]
;;;		: Transporter-switchable (and its jammer children) now
;;;		uses the new info2 slot of the proc-action class of 
;;;		procedure-window2 [now r6]. This allows the subtask order
;;;		information from those windows to be recorded for each 
;;;		step-action in the d2protocol file.
;;; 2008.03.16 fpt [r2]
;;;		: created transporter-window-switchable class, which can 
;;;		allow for two different orderings of the frequency and
;;;		power subtasks, depending upon whether the user clicks
;;;		"enter frequency" or "transporter power" on step #5.
;;; 2008.03.06 fpt [r1]
;;;		: inception
;;;		: Altered do-button to throw up a warning message about exactly 
;;;		how the task changes.
;;;		: Added transporter/jammer-same/jammer-different slots to
;;;		the main control window so that a list of such trials
;;;		can be presented for any instance of a generic trial,
;;;		e.g., so either of two transporter particular windows can be 
;;;		presented when it's time for a transporter window.
;;;
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(require :quickdraw)
(load "dissertation-model:base-trek-tasks-actr.lisp")
(load "dissertation-model:experiment-window4-actr.lisp")
(require :blinker)



;; (require :menubar-hide)                 ; uncomment before deploy

(defparameter *res-file*
  (open-resource-file (aif (probe-file "ccl:alias;X84.rsrc")
                        it
                        (choose-file-dialog :prompt "Resource file?"))))

(defparameter *phaser-speed* 3)
(defparameter *transporter-speed* 8)
(defparameter *exp-cond* nil)


#|
Experiment conditions (and task IDs):
0: 
Phaser: trained interruption (0)
Transporter: jammer trained reordering(500)

1: 
Phaser: untrained interruption (0, 100)
Transporter: jammer untrained reordering(600, 500)

2:
Phaser: no interruption (100)
Transporter: jammer-same (600)

3:
Phaser: semantic control (300)
Transporter static subgoal order (700)
new jammer static subgoal order  (800)

Transporter (400)
Nav (200)

|#



;;;; ---------------------------------------------------------------------- ;;;;


(defclass phaser-window-interruption (phaser-window)
  ;; (setf *experiment* (make-instance 'phaser-window-interruption :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :task-id 0
    :pict-name "phaser"
    :window-title "Phaser Intervening Subtask"
    :trgt-cntr #@(260 230)
    :state-vec #(:POWER-CONNECTED :CHARGE :SETTINGS :SLIDER :FOCUS-SET 
                                  :STOP-CHARGING :POWER-CONNECTED 
                                  :FIRING 
                                  :TRACKING :SHOOT :TRACKING :MAIN-CONTROL)
    :gui-vec #((check-box-check . :POWER-CONNECTED) nil nil nil nil nil
               (check-box-uncheck . :POWER-CONNECTED) (radio-button-push . :FIRING)
               nil nil nil nil)
    
    :view-subviews
    (list 
     ;; tracking cluster
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(275 392)
      #@(110 30)
      "Shot"
      nil
      :VIEW-NICK-NAME :TRACKING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; power cluster
     (make-dialog-item
      'button-checker
      #@(680 125)
      #@(120 34)
      "Generator"
      nil
      :VIEW-NICK-NAME :CHARGE
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(680 260)
      #@(120 34)
      "Kilowatts"
      nil
      :VIEW-NICK-NAME :STOP-CHARGING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     (make-instance 'thermometer 
       :view-position #@(878 119)
       :view-size #@(25 174)
       :view-nick-name :THERMO
       :pattern *light-gray-pattern*)
     
     
     ;; central button cluster
#|
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(-100 342)
      #@(70 15)
      "Battery"
      nil
      :VIEW-NICK-NAME :BATTERY
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
|#
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(145 401)
      #@(70 16)
      "Cannon"
      nil
      :VIEW-NICK-NAME :FIRING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(475 595)
      #@(70 16)
      "Lens"
      nil
      :VIEW-NICK-NAME :SETTINGS
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(600 595)
      #@(70 16)
      "Focus"
      nil
      :VIEW-NICK-NAME :FOCUS-SET
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(700 200)
      #@(70 16)
      "Electrical"
      nil
      :VIEW-NICK-NAME :POWER-connected
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     
     
     ;; main control
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(794 589)
      #@(108 34)
      "Main"
      nil
      :VIEW-NICK-NAME :MAIN-CONTROL
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; focus
     (make-instance 'click-indicator :view-position #@(475 637)
                    :view-size #@(240 44) :v-offset 24
                    :view-nick-name :SLIDER)
     
     ;; elapsed time
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(826 667)
      #@(54 14)
      "0"
      'NIL
      :VIEW-NICK-NAME :TIME
      :VIEW-FONT
      '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     
     ;; status
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(119 644)
      #@(273 38)
      ""
      'NIL
      :VIEW-NICK-NAME :STATUS
      :VIEW-FONT '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     )
    ))

(defmethod check-state-update ((wind phaser-window-interruption) state)
  (case state
    (:POWER-CONNECTED (status wind "") (view-draw-contents wind))
    (:FOCUS-SET (setf (charge-p wind) t))
    (:STOP-CHARGING (setf (charge-p wind) nil))
    (:TRACKING (if (= (state-num wind) 8)
                 (tracking-on wind)
                 (tracking-off wind)))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))


(defclass phaser-window-no-interruption (phaser-window)
  ;; (setf *experiment* (make-instance 'phaser-window-no-interruption :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :task-id 100
    :pict-name "phaser"
    :window-title "Phaser No Interruption"
    :trgt-cntr #@(260 230)
    :view-subviews
    (list 
     ;; tracking cluster
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(275 392)
      #@(110 30)
      "Shot"
      nil
      :VIEW-NICK-NAME :TRACKING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; power cluster
     (make-dialog-item
      'button-checker
      #@(680 125)
      #@(120 34)
      "Generator"
      nil
      :VIEW-NICK-NAME :CHARGE
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(680 260)
      #@(120 34)
      "Kilowatts"
      nil
      :VIEW-NICK-NAME :STOP-CHARGING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     (make-instance 'thermometer 
       :view-position #@(878 119)
       :view-size #@(25 174)
       :view-nick-name :THERMO
       :pattern *light-gray-pattern*)
     
     
     ;; central button cluster
#|
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(-100 342)
      #@(70 15)
      "Battery"
      nil
      :VIEW-NICK-NAME :BATTERY
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
|#
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(145 401)
      #@(70 16)
      "Cannon"
      nil
      :VIEW-NICK-NAME :FIRING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(475 595)
      #@(70 16)
      "Lens"
      nil
      :VIEW-NICK-NAME :SETTINGS
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(600 595)
      #@(70 16)
      "Focus"
      nil
      :VIEW-NICK-NAME :FOCUS-SET
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(700 200)
      #@(90 16)
      "Electrical"
      nil
      :VIEW-NICK-NAME :POWER-connected
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     
     
     ;; main control
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(794 589)
      #@(108 34)
      "Main"
      nil
      :VIEW-NICK-NAME :MAIN-CONTROL
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; focus
     (make-instance 'click-indicator :view-position #@(475 637)
                    :view-size #@(240 44) :v-offset 24
                    :view-nick-name :SLIDER)
     
     ;; elapsed time
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(826 667)
      #@(54 14)
      "0"
      'NIL
      :VIEW-NICK-NAME :TIME
      :VIEW-FONT
      '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     
     ;; status
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(119 644)
      #@(273 38)
      ""
      'NIL
      :VIEW-NICK-NAME :STATUS
      :VIEW-FONT '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     )
    ))

#|
(defmethod initialize-instance :after ((wind phaser-window-no-interruption) &key)
;;  (set-view-pict wind (pict-name wind))
  (start-timing (timer wind))
  (set-cursor *arrow-cursor*)
  (mapc #'view-draw-contents (subviews wind))
  (view-draw-contents wind)
  (when *actr-enabled-p*
    (install-device wind)
    (proc-display :clear t))
  )
|#




#|
(defmethod shot-feedback ((wind phaser-window))
  (cond ((= (num-hits wind) -1)
         (status wind "Romulan vessel destroyed.")
         (unless *actr-enabled-p* (play-snd *sp* "explosion"))
         (unless *actr-enabled-p* (play-snd *sp* "explosion" t))
         )
        ((last-hit-p wind)
         (unless *actr-enabled-p* (play-snd *sp* "explosion" t))
         (status wind "Romulan vessel hit but not destroyed.")
         (setf (track-p wind) nil)
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil))
        (t
         (unless *actr-enabled-p* (play-snd *sp* "Whip"))
         (setf (track-p wind) nil)
         (status wind "Phaser missed Romulan vessel.")
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)
         )))
|#


(defclass phaser-window-semantic-control (phaser-window)
  ;; (setf *experiment* (make-instance 'phaser-window-semantic-control :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :task-id 300
    :pict-name "phaser"
    :window-title "Phaser Semantic Control"
    :trgt-cntr #@(260 230)
    :view-subviews
    (list 
     ;; tracking cluster
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(275 392)
      #@(110 30)
      "Headings"
      nil
      :VIEW-NICK-NAME :TRACKING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; power cluster
     (make-dialog-item
      'button-checker
      #@(680 125)
      #@(120 34)
      "Entomologist"
      nil
      :VIEW-NICK-NAME :CHARGE
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(680 260)
      #@(120 34)
      "Filmmaker"
      nil
      :VIEW-NICK-NAME :STOP-CHARGING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     (make-instance 'thermometer 
       :view-position #@(878 119)
       :view-size #@(25 174)
       :view-nick-name :THERMO
       :pattern *light-gray-pattern*)
     
     
     ;; central button cluster
#|
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(-100 342)
      #@(70 15)
      "Battery"
      nil
      :VIEW-NICK-NAME :BATTERY
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
|#
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(145 401)
      #@(90 16)
      "Redcoats"
      nil
      :VIEW-NICK-NAME :FIRING
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'radio-checker
      #@(475 595)
      #@(70 16)
      "Solenoid"
      nil
      :VIEW-NICK-NAME :SETTINGS
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(600 595)
      #@(90 16)
      "Reflecting"
      nil
      :VIEW-NICK-NAME :FOCUS-SET
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     (MAKE-DIALOG-ITEM
      'check-box-checker
      #@(700 200)
      #@(70 16)
      "Smokey"
      nil
      :VIEW-NICK-NAME :POWER-connected
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0)))
     
     
     ;; main control
     (MAKE-DIALOG-ITEM
      'button-checker
      #@(794 589)
      #@(108 34)
      "Drafty"
      nil
      :VIEW-NICK-NAME :MAIN-CONTROL
      :VIEW-FONT '("Avant Garde" 12 :SRCCOPY :PLAIN (:COLOR-INDEX 0))
      :DEFAULT-BUTTON NIL)
     
     ;; focus
     (make-instance 'click-indicator :view-position #@(475 637)
                    :view-size #@(240 44) :v-offset 24
                    :view-nick-name :SLIDER)
     
     ;; elapsed time
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(826 667)
      #@(54 14)
      "0"
      'NIL
      :VIEW-NICK-NAME :TIME
      :VIEW-FONT
      '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     
     ;; status
     (MAKE-DIALOG-ITEM
      'STATIC-TEXT-DIALOG-ITEM
      #@(119 644)
      #@(273 38)
      ""
      'NIL
      :VIEW-NICK-NAME :STATUS
      :VIEW-FONT '("Avant Garde" 14 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     )
    ))




;;;; ---------------------------------------------------------------------- ;;;;
;;;; Transporter window variants


(defclass transporter-window-switchable (transporter-window)
  ;; (setf *experiment* (make-instance 'transporter-window-switchable :snd-player *sp* :view-position #@(50 50)))
  ((subtask-ordering :accessor subtask-ordering :initarg :subtask-ordering :initform 0)
   (scan-rate :accessor scan-rate :initarg :scan-rate :initform 100)
   (scan-count :accessor scan-count :initarg :scan-count :initform 2))
  ;; 0 means freq-first, 1 means power-first
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Transporter Switchable"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 400
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))


(defmethod state-check ((wind transporter-window-switchable) state-name 
                          &optional info)
  (let ((p-act (make-instance 'proc-action
                  :latency (round (start-stop-timer (timer wind)))
                  ;;                  :expected (curr-state wind)
                  ;; if (curr-state wind) = :enter-frequency and state-name = :transporter-power and state-num = 4,
                  ;; then set :expected to state-name and call change-procedure-to-pow,
                  ;; or if (curr-state) wind = :transporter-power and state-name = :enter-frequency and state-num = 4,
                  ;; then set :expected to state-name and call change-procedure-to-freq,
                  ;; else set :expected to (curr-state wind)
                  :expected (cond
                             ((and (eq (curr-state wind) :enter-frequency)
                                   (eq state-name :transporter-power)
                                   (= (state-num wind) 4))
                              (progn (change-procedure-to-pow wind) state-name))
                             ((and (eq (curr-state wind) :transporter-power)
                                   (eq state-name :enter-frequency)
                                   (= (state-num wind) 4))
                              (progn (change-procedure-to-freq wind) state-name))
                             (t (curr-state wind)))
                  :got state-name
                  :step-num (state-num wind)
                  :info info
                  :info2 (subtask-ordering wind))))
;;    (format t "~a	~a" state-name (subtask-ordering wind))
    (push p-act (action-log wind))
;;    (push (subtask-ordering wind) (subtask-order-log wind))
    (if (not (is-error p-act))
      (advance-state wind)
      (proc-error wind))))

(defmethod change-procedure-to-pow ((wind transporter-window-switchable))
  ;;  (format t "switching subtask ordering for ~A!~%" wind))
  ;;  (setf (task-id wind) 1400)
  (setf (subtask-ordering wind) 1)
  (setf (state-vec wind) #(:scanner-on :active-scan :lock-signal :scanner-off
                                       :transporter-power :synchronous-mode
                                       :enter-frequency :type :accept-frequency
                                       :shoot :synchronous-mode :main-control))
  (setf (gui-vec wind) #((radio-button-push . :scanner-on) 
                         (radio-button-push . :active-scan)
                         (radio-button-push . :lock-signal)
                         (radio-button-push . :scanner-off)
                         (check-box-check . :transporter-power) nil 
                         nil nil (check-box-check . :accept-frequency)
                         nil nil nil)))

(defmethod change-procedure-to-freq ((wind transporter-window-switchable))
  ;;  (format t "switching subtask ordering for ~A!~%" wind))
  ;;  (setf (task-id wind) 400)
  (setf (subtask-ordering wind) 0)
  (setf (state-vec wind) #(:scanner-on :active-scan :lock-signal :scanner-off
                                       :enter-frequency :type :accept-frequency
                                       :transporter-power :synchronous-mode
                                       :shoot :synchronous-mode :main-control))
  (setf (gui-vec wind) #((radio-button-push . :scanner-on) 
                         (radio-button-push . :active-scan)
                         (radio-button-push . :lock-signal)
                         (radio-button-push . :scanner-off) nil nil 
                         (check-box-check . :accept-frequency) 
                         (check-box-check . :transporter-power) nil nil nil nil)))



(defmethod initialize-instance :after ((wind transporter-window-switchable)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (when *actr-enabled-p*
    (install-device wind)
    (proc-display :clear t))
  )

(defmethod reset-display ((wind transporter-window-switchable))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  #|
  (setf (task-id wind) 400)
  (setf (subtask-ordering wind) 0)
  (setf (state-vec wind) #(:scanner-on :active-scan :lock-signal :scanner-off
                                       :enter-frequency :type :accept-frequency
                                       :transporter-power :synchronous-mode
                                       :shoot :synchronous-mode :main-control))
  (setf (gui-vec wind) #((radio-button-push . :scanner-on) 
                         (radio-button-push . :active-scan)
                         (radio-button-push . :lock-signal)
                         (radio-button-push . :scanner-off) nil nil 
                         (check-box-check . :accept-frequency) 
                         (check-box-check . :transporter-power) nil nil nil nil))
|#
  )

(defmethod check-state-update ((wind transporter-window-switchable) state)
  ;(print state)
  (case (subtask-ordering wind)
    (0 (case state
         (:scanner-on (status wind ""))
         (:active-scan (setf (scan-p wind) t))
         (:lock-signal (setf (scan-p wind) nil))
         (:enter-frequency (activate-freq wind))
         ;;         (:transporter-power (format t "transporter-power was clicked!~%"))
         (:synchronous-mode (if (eq (state-num wind) 8)
                              (begin-tracking wind)
                              (unless (postcomp-p wind) (give-feedback wind))))
         (:accept-frequency (check-freq wind))
         (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))))
    (1   (case state
           (:scanner-on (status wind ""))
           (:active-scan (setf (scan-p wind) t))
           (:lock-signal (setf (scan-p wind) nil))
           (:enter-frequency (activate-freq wind))
           (:accept-frequency (check-freq wind) (begin-tracking wind))
           (:synchronous-mode (if (eq (state-num wind) 10)
                                (unless (postcomp-p wind) (give-feedback wind))))
           (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
           ))))
#|
(defmethod write-events ((wind transporter-window-switchable) &optional (strm t))
  (setf *tid* (task-id wind))
;;  (setf *s-o-log* (reverse (subtask-order-log wind)))
  (terpri)
  (dolist (so-log (reverse (subtask-order-log wind)))
    (format t "~A~%" so-log))
;;  (terpri)
;;  (format t "new event~%")
  (multiple-value-bind (sec min hour date mnth yr)
                       (decode-universal-time (get-universal-time))
    (declare (ignore sec))
    (format strm "~%---------- Task ID: ~S  trial ending ~S:~S on ~S/~S/~S ----------"
            (task-id wind) hour min mnth date yr)
    (dolist (p-act (reverse (action-log wind)))
      (write-pa p-act strm)
      ))
  ;; (terpri)
  ;; (format t "~A" (reverse (subtask-order-log wind)))
  )
|#


;; for running with ACT-R
(defmethod move-target ((wind transporter-window-switchable) (max-dist fixnum))
  nil)

#|
(defmethod cease-tracking ((wind transporter-window))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind))
  )

(defmethod give-feedback ((wind transporter-window))
  (cond ((last-hit-p wind)
;;         (play-snd *sp* "yes" t)
         (status wind "Beam successful--return to main control.")
         )
        (t
;;         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Beam failed--jammed by hostile signal."
                        "Beam failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))
|#

(defmethod view-click-event-handler :after ((wind transporter-window) where)
  (declare (ignore where))
  (when (and (< (state-num wind) 10) (eq (curr-state wind) :shoot))
    (unless *actr-enabled-p* (play-snd *sp* "whit" t))))





(defclass transporter-window-order0 (transporter-window)
  ;; (setf *experiment* (make-instance 'transporter-window-order0 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Transporter Order 0"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 400
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))



(defmethod initialize-instance :after ((wind transporter-window-order0)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Primary Subtask Order"))
  )

(defmethod reset-display ((wind transporter-window-order0))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )


(defclass transporter-window-order1 (transporter-window)
  ;; (setf *experiment* (make-instance 'transporter-window-order1 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Transporter Order 1"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 1400
    :state-vec #(:scanner-on :active-scan :lock-signal :scanner-off
                             :transporter-power :synchronous-mode
                             :enter-frequency :type :accept-frequency
                             :shoot :synchronous-mode :main-control)
    :gui-vec #((radio-button-push . :scanner-on) 
               (radio-button-push . :active-scan)
               (radio-button-push . :lock-signal)
               (radio-button-push . :scanner-off)
               (check-box-check . :transporter-power) nil 
               nil nil (check-box-check . :accept-frequency)
               nil nil nil)
    
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))




(defmethod initialize-instance :after ((wind transporter-window-order1)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Secondary Subtask Order"))
  )

(defmethod check-state-update ((wind transporter-window-order1) state)
  ;(print state)
  (case state
    (:scanner-on (status wind ""))
    (:active-scan (setf (scan-p wind) t))
    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:accept-frequency (check-freq wind) (begin-tracking wind))
    (:synchronous-mode (if (eq (state-num wind) 10)
                         (unless (postcomp-p wind) (give-feedback wind))
                         ;;(begin-tracking wind)
                         ))
    
    (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))

(defmethod reset-display ((wind transporter-window-order1))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )



(defclass transporter-window-static-subtask-order (transporter-window)
  ;; (setf *experiment* (make-instance 'transporter-window-static-subtask-order :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Transporter Static Subgoal Order"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 700
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))



(defmethod initialize-instance :after ((wind transporter-window-static-subtask-order)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  )

(defmethod reset-display ((wind transporter-window-static-subtask-order))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 1)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 1)
  )


(defclass jammer-window-1 (transporter-window-static-subtask-order)
  ;; (setf *experiment* (make-instance 'transporter-window-static-subtask-order :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :window-title "Jammer Same"
    :task-id 600
     ))





(defclass jammer-window-0 (transporter-window-static-subtask-order)
  ;; (setf *experiment* (make-instance 'jammer-window-static-subtask-order :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :window-title "Jammer Different Scanner"
    :task-id 500
    :state-vec #(:active-scan :scanner-off :lock-signal :scanner-on
                              :enter-frequency :type :accept-frequency
                              :transporter-power :synchronous-mode
                              :shoot :synchronous-mode :main-control)
    :gui-vec #(nil nil nil 
                   (radio-button-push . :scanner-on) 
                   (radio-button-push . :active-scan)
                   (radio-button-push . :lock-signal)
                   (radio-button-push . :scanner-off) 
                   (check-box-check . :transporter-power) nil nil nil nil)
    ))

(defmethod check-state-update ((wind jammer-window-0) state)
  ;(print state)
  (case state
    (:active-scan (status wind ""))
;;    (:active-scan (setf (scan-p wind) t))
;;    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:synchronous-mode (if (eq (state-num wind) 8)
                         (begin-tracking wind)
                         (unless (postcomp-p wind) (give-feedback wind))))
    (:accept-frequency (check-freq wind))
    (:shoot (cease-tracking wind))
    (:MAIN-CONTROL (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))



(defclass jammer-window-static-subtask-order (transporter-window-static-subtask-order)
  ;; (setf *experiment* (make-instance 'jammer-window-static-subtask-order :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Static Subgoal Order"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 800
    :state-vec #(:enter-frequency :type :accept-frequency
                                  :scanner-on :active-scan :lock-signal :scanner-off
                                  :transporter-power :synchronous-mode
                                  :shoot :synchronous-mode :main-control)
    :gui-vec #(nil nil (check-box-check . :accept-frequency) 
                   (radio-button-push . :scanner-on) 
                   (radio-button-push . :active-scan)
                   (radio-button-push . :lock-signal)
                   (radio-button-push . :scanner-off) 
                   (check-box-check . :transporter-power) nil nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))

#|
(defmethod give-feedback ((wind jammer-window-static-subtask-order))
  (cond ((last-hit-p wind)
;;         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
;;         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-static-subtask-order))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
;;  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind)))
|#







(defclass jammer-window-different-switchable (transporter-window-switchable)
  ;; (setf *experiment* (make-instance 'jammer-window-different-switchable :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Different Switchable"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 500
    :state-vec #(:active-scan :scanner-off :lock-signal :scanner-on
                              :enter-frequency :type :accept-frequency
                              :transporter-power :synchronous-mode
                              :shoot :synchronous-mode :main-control)
    :gui-vec #((radio-button-push . :active-scan)
               (radio-button-push . :scanner-off)
               (radio-button-push . :lock-signal) 
               (radio-button-push . :scanner-on)
               nil nil 
               (check-box-check . :accept-frequency) 
               (check-box-check . :transporter-power) nil nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))


#|
(defmethod initialize-instance :after ((wind jammer-window-different-switchable)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
;;  (warning (format nil "Primary Subtask Order"))
  )
|#

(defmethod give-feedback ((wind jammer-window-different-switchable))
  (cond ((last-hit-p wind)
         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-different-switchable))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind)))

#|
(defmethod check-state-update ((wind jammer-window-different-switchable) state)
  ;(print state)
  (case state
    (:active-scan (status wind ""))
    (:scanner-off (setf (scan-p wind) t))
    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:synchronous-mode (if (eq (state-num wind) 8)
                         (begin-tracking wind)
                         (unless (postcomp-p wind) (give-feedback wind))))
    (:accept-frequency (check-freq wind))
    (:shoot (cease-tracking wind))
    (:main-control (finish-task wind))
    ))

(defmethod reset-display ((wind jammer-window-different-switchable))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )

(defmethod state-check ((wind jammer-window-different-switchable) state-name 
                          &optional info)
  (let ((p-act (make-instance 'proc-action
                  :latency (round (start-stop-timer (timer wind)))
                  ;;                  :expected (curr-state wind)
                  ;; if (curr-state wind) = :enter-frequency and state-name = :transporter-power and state-num = 4,
                  ;; then set :expected to state-name and call change-procedure-to-pow,
                  ;; or if (curr-state) wind = :transporter-power and state-name = :enter-frequency and state-num = 4,
                  ;; then set :expected to state-name and call change-procedure-to-freq,
                  ;; else set :expected to (curr-state wind)
                  :expected (cond
                             ((and (eq (curr-state wind) :enter-frequency)
                                   (eq state-name :transporter-power)
                                   (= (state-num wind) 4))
                              (progn (change-procedure-to-pow wind) state-name))
                             ((and (eq (curr-state wind) :transporter-power)
                                   (eq state-name :enter-frequency)
                                   (= (state-num wind) 4))
                              (progn (change-procedure-to-freq wind) state-name))
                             (t (curr-state wind)))
                  :got state-name
                  :step-num (state-num wind)
                  :info info)))
    (push p-act (action-log wind))
    (if (not (is-error p-act))
      (advance-state wind)
      (proc-error wind))))
|#

(defmethod change-procedure-to-pow ((wind jammer-window-different-switchable))
  ;;  (format t "switching subtask ordering for ~A!~%" wind))
  ;;  (setf (task-id wind) 1400)
  (setf (subtask-ordering wind) 1)
  (setf (state-vec wind) #(:active-scan :scanner-off :lock-signal :scanner-on
                                        :transporter-power :synchronous-mode
                                        :enter-frequency :type :accept-frequency
                                        :shoot :synchronous-mode :main-control))
  (setf (gui-vec wind) #((radio-button-push . :active-scan)
                         (radio-button-push . :scanner-off)
                         (radio-button-push . :lock-signal) 
                         (radio-button-push . :scanner-on)
                         (check-box-check . :transporter-power) nil 
                         nil nil (check-box-check . :accept-frequency)
                         nil nil nil)))

(defmethod change-procedure-to-freq ((wind jammer-window-different-switchable))
  ;;  (format t "switching subtask ordering for ~A!~%" wind))
  ;;  (setf (task-id wind) 400)
  (setf (subtask-ordering wind) 0)
  (setf (state-vec wind) #(:active-scan :scanner-off :lock-signal :scanner-on
                                        :enter-frequency :type :accept-frequency
                                        :transporter-power :synchronous-mode
                                        :shoot :synchronous-mode :main-control))
  (setf (gui-vec wind) #((radio-button-push . :active-scan)
                         (radio-button-push . :scanner-off)
                         (radio-button-push . :lock-signal) 
                         (radio-button-push . :scanner-on)
                         nil nil 
                         (check-box-check . :accept-frequency) 
                         (check-box-check . :transporter-power) nil nil nil nil)))






(defclass jammer-window-different-order0 (transporter-window)
  ;; (setf *experiment* (make-instance 'jammer-window-different-order0 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Different Order 0"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 500
    :state-vec #(:active-scan :scanner-off :lock-signal :scanner-on
                              :enter-frequency :type :accept-frequency
                              :transporter-power :synchronous-mode
                              :shoot :synchronous-mode :main-control)
    :gui-vec #((radio-button-push . :active-scan)
               (radio-button-push . :scanner-off)
               (radio-button-push . :lock-signal) 
               (radio-button-push . :scanner-on)
               nil nil 
               (check-box-check . :accept-frequency) 
               (check-box-check . :transporter-power) nil nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))



(defmethod initialize-instance :after ((wind jammer-window-different-order0)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Primary Subtask Order"))
  )

(defmethod give-feedback ((wind jammer-window-different-order0))
  (cond ((last-hit-p wind)
         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-different-order0))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind))
  )

(defmethod check-state-update ((wind jammer-window-different-order0) state)
  ;(print state)
  (case state
    (:active-scan (status wind ""))
    (:scanner-off (setf (scan-p wind) t))
    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:synchronous-mode (if (eq (state-num wind) 8)
                         (begin-tracking wind)
                         (unless (postcomp-p wind) (give-feedback wind))))
    (:accept-frequency (check-freq wind))
    (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))

(defmethod reset-display ((wind jammer-window-different-order0))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )





(defclass jammer-window-different-order1 (transporter-window)
  ;; (setf *experiment* (make-instance 'jammer-window-different-order1 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Different Order 1"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 1500
    :state-vec #(:active-scan :scanner-off :lock-signal :scanner-on
                              :transporter-power :synchronous-mode
                              :enter-frequency :type :accept-frequency
                              :shoot :synchronous-mode :main-control)
    :gui-vec #( 
               (radio-button-push . :active-scan) 
               (radio-button-push . :scanner-off)
               (radio-button-push . :lock-signal)
               (radio-button-push . :scanner-on)
               (check-box-check . :transporter-power) nil 
               nil nil (check-box-check . :accept-frequency)
               nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))

(defmethod initialize-instance :after ((wind jammer-window-different-order1)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Secondary Subtask Order"))
  )

(defmethod check-state-update ((wind jammer-window-different-order1) state)
  ;(print state)
  (case state
    (:active-scan (status wind ""))
    (:scanner-off (setf (scan-p wind) t))
    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:accept-frequency (check-freq wind) (begin-tracking wind))
    (:synchronous-mode (if (eq (state-num wind) 10)
                         (unless (postcomp-p wind) (give-feedback wind))
                         ;;(begin-tracking wind)
                         ))
    
    (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))

(defmethod give-feedback ((wind jammer-window-different-order1))
  (cond ((last-hit-p wind)
         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-different-order1))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind))
  )

(defmethod reset-display ((wind jammer-window-different-order1))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )



(defclass jammer-window-same-switchable (transporter-window-switchable)
  ;; (setf *experiment* (make-instance 'jammer-window-same-switchable :snd-player *sp* :view-position #@(50 50)))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Same Switchable"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 600
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))

(defmethod initialize-instance :after ((wind jammer-window-same-switchable)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (when *actr-enabled-p*
    (install-device wind)
    (proc-display :clear t))
  )



(defmethod give-feedback ((wind jammer-window-same-switchable))
  (cond ((last-hit-p wind)
         (unless *actr-enabled-p* (play-snd *sp* "yes" t))
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (unless *actr-enabled-p* (play-snd *sp* "damn" t))
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-same-switchable))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (unless *actr-enabled-p* (play-snd *sp* "Jammer" t))
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind)))





(defclass jammer-window-same-order0 (transporter-window)
  ;; (setf *experiment* (make-instance 'jammer-window-same-order0 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Same Order 0"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 700
    :state-vec #(:scanner-on :active-scan :lock-signal :scanner-off
                             :enter-frequency :type :accept-frequency
                             :transporter-power :synchronous-mode
                             :shoot :synchronous-mode :main-control)
    :gui-vec #((radio-button-push . :scanner-on) 
               (radio-button-push . :active-scan)
               (radio-button-push . :lock-signal)
               (radio-button-push . :scanner-off) nil nil 
               (check-box-check . :accept-frequency) 
               (check-box-check . :transporter-power) nil nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))



(defmethod initialize-instance :after ((wind jammer-window-same-order0)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Primary Subtask Order"))
  )

(defmethod give-feedback ((wind jammer-window-same-order0))
  (cond ((last-hit-p wind)
         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-same-order0))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind))
  )

(defmethod reset-display ((wind jammer-window-same-order0))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )




(defclass jammer-window-same-order1 (transporter-window)
  ;; (setf *experiment* (make-instance 'jammer-window-same-order1 :snd-player *sp*))
  ()
  (:default-initargs
    :pict-name "transporter-extra-buttons"
    :window-title "Jammer Same Order 1"
    :dots-center #@(752 173)
    :x1-bound 120
    :x2-bound 412
    :y1-bound 352
    :y2-bound 645
    :task-id 1700
    :state-vec #(:scanner-on :active-scan :lock-signal :scanner-off
                             :transporter-power :synchronous-mode
                             :enter-frequency :type :accept-frequency
                             :shoot :synchronous-mode :main-control)
    :gui-vec #( 
               (radio-button-push . :scanner-on) 
               (radio-button-push . :active-scan)
               (radio-button-push . :lock-signal)
               (radio-button-push . :scanner-off)
               (check-box-check . :transporter-power) nil 
               nil nil (check-box-check . :accept-frequency)
               nil nil nil)
    :view-subviews
    (list
     ;; left button grouping
     (make-dialog-item
      'button-checker
      #@(185 107)
      #@(150 28)
      "Desynchronize"
      'nil
      :view-nick-name :DESYNCHRONIZE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(185 145)
      #@(150 28)
      "Synchronous Mode"
      'nil
      :view-nick-name :SYNCHRONOUS-MODE
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'check-box-checker
      #@(189 223)
      #@(151 16)
      "Transporter Power"
      'nil
      :view-nick-name :TRANSPORTER-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 244)
      #@(151 16)
      "Diagnostic Z"
      'nil
      :view-nick-name :DIAGNOSTIC-Z
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     (make-dialog-item
      'check-box-checker
      #@(189 265)
      #@(151 16)
      "Auxiliary Power"
      'nil
      :view-nick-name :AUXILIARY-POWER
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))      
     
     ;; Middle button group
     (make-dialog-item
      'button-checker
      #@(440 107)
      #@(150 28)
      "Calibration"
      'nil
      :view-nick-name :ENTER-CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'button-checker
      #@(440 145)
      #@(150 28)
      "Enter Frequency"
      'nil
      :view-nick-name :ENTER-FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :default-button nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(525 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :CALIBRATION
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'editable-text-dialog-item
      #@(463 188)
      #@(35 15)
      "0"
      'nil
      :view-nick-name :FREQUENCY
      :view-font '("Avant Garde" 14 :srccopy :plain (:color-index 0))
      :allow-returns nil
      :draw-outline t :enabled-p nil)
     (make-dialog-item
      'check-box-checker
      #@(439 224)
      #@(151 16)
      "Accept Frequency"
      'nil
      :view-nick-name :ACCEPT-FREQUENCY
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 245)
      #@(151 16)
      "Accept Calibration"
      'nil
      :view-nick-name :ACCEPT-CALIBRATION
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'check-box-checker
      #@(439 266)
      #@(151 16)
      "Frequency Sample"
      'nil
      :view-nick-name :FREQ-SAMPLE
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     ;; main control group
     (make-dialog-item 'button-checker
                       #@(505 577) #@(108 34)
                       "Main Control" nil :view-nick-name :MAIN-CONTROL
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 577)
                       #@(108 34)
                       "Diagnostic" nil :view-nick-name :DIAGNOSTIC
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(505 617)
                       #@(108 34)
                       "Alpha Mask" nil :view-nick-name :ALPHA-MASK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     (make-dialog-item 'button-checker
                       #@(625 617)
                       #@(108 34)
                       "System Lock" nil :view-nick-name :SYSTEM-LOCK
                       :view-font '("Charcoal" 12 :srccopy :plain (:color-index 0))
                       :default-button NIL)
     
     ;; right button group -- out of order to get defaults correct
     (make-dialog-item
      'radio-checker
      #@(703 281)
      #@(117 16)
      "Scanner Off"
      'nil
      :view-nick-name :SCANNER-OFF
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 260)
      #@(117 16)
      "Scanner On"
      'nil
      :view-nick-name :SCANNER-ON
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 323)
      #@(117 16)
      "Lock Signal"
      'nil
      :view-nick-name :LOCK-SIGNAL
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 302)
      #@(117 16)
      "Active Scan"
      'nil
      :view-nick-name :ACTIVE-SCAN
      :radio-button-cluster 1
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 365)
      #@(117 16)
      "Tracker Off"
      'nil
      :view-nick-name :TRACKER-OFF
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 344)
      #@(117 16)
      "Tracker On"
      'nil
      :view-nick-name :TRACKER-ON
      :radio-button-cluster 2
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     (make-dialog-item
      'radio-checker
      #@(703 407)
      #@(117 16)
      "Fix Frequency"
      'nil
      :view-nick-name :FIX-FREQUENCY
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'radio-checker
      #@(703 386)
      #@(117 16)
      "Modulator"
      'nil
      :view-nick-name :MODULATOR
      :radio-button-cluster 3
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     
     
     (make-dialog-item
      'static-text-dialog-item
      #@(529 492)
      #@(274 37)
      ""
      'nil
      :view-nick-name :STATUS
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     (make-dialog-item
      'static-text-dialog-item
      #@(834 635)
      #@(32 15)
      "0"
      'nil
      :view-nick-name :TIME
      :view-font '("Avant Garde" 14 :srcor :plain (:color-index 0)))
     )))

(defmethod initialize-instance :after ((wind jammer-window-same-order1)
                                           &key)
  (dialog-item-disable (view-named :calibration wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (warning (format nil "Secondary Subtask Order"))
  )

(defmethod check-state-update ((wind jammer-window-same-order1) state)
  ;(print state)
  (case state
    (:scanner-on (status wind ""))
    (:active-scan (setf (scan-p wind) t))
    (:lock-signal (setf (scan-p wind) nil))
    (:enter-frequency (activate-freq wind))
    (:accept-frequency (check-freq wind) (begin-tracking wind))
    (:synchronous-mode (if (eq (state-num wind) 10)
                         (unless (postcomp-p wind) (give-feedback wind))
                         ;;(begin-tracking wind)
                         ))
    
    (:shoot (cease-tracking wind))
    (:main-control (prog1
                     (setf (advance-p wind) nil)
                     (finish-task wind)))
    ))

(defmethod give-feedback ((wind jammer-window-same-order1))
  (cond ((last-hit-p wind)
         (play-snd *sp* "yes" t)
         (status wind "Jamming successful--return to main control.")
         )
        (t
         (play-snd *sp* "damn" t)
         (status wind (if (flip)
                        "Jamming failed--hostile signal too strong."
                        "Jamming failed--beam too weak."))
         (setf (state-num wind) 0)
         (setf (advance-p wind) nil)))
  (unless (postcomp-p wind)
    (reset-display wind)))

(defmethod cease-tracking ((wind jammer-window-same-order1))
  (remove-subviews wind (trgt-v wind))
  (setf (track-p wind) nil)
  (play-snd *sp* "Jammer" t)
  (set-cursor *arrow-cursor*)
  (determine-hit wind)
  (when (postcomp-p wind) 
    (give-feedback wind)
    (reset-display wind))
  )

(defmethod reset-display ((wind jammer-window-same-order1))
  (check-box-uncheck (view-named :accept-frequency wind))
  (check-box-uncheck (view-named :transporter-power wind))
  (radio-button-unpush (view-named :scanner-off wind))
  (radio-button-unpush (view-named :lock-signal wind))
  (radio-button-unpush (view-named :tracker-off wind))
  (radio-button-unpush (view-named :scanner-on wind))
  (radio-button-unpush (view-named :fix-frequency wind))
  (set-dialog-item-text (view-named :frequency wind) "0")
  (dotimes (i 4)
    (awhen (scan-marker wind (1+ i))
      (remove-subviews wind it)))
  (add-markers wind)
  (setf (scan-count wind) 4)
  )





;;;; ---------------------------------------------------------------------- ;;;;
;;;;
;;;; Main control
;;;;
;;;; ---------------------------------------------------------------------- ;;;;

(defun knum-key (knum)
  (rest (assoc knum '((0 . :PHASER) (1 . :TRANSPORTER) (2 . :NAVIGATION)
                      (3 . :JAMMER)))))

(defclass x84-trial (trial)
  ((rt :accessor rt :initarg :rt :initform nil)
   (nerrors :accessor nerrors :initarg :nerrors :initform nil)
   (xcond :accessor xcond :initarg :xcond :initform nil)
   (train-p :accessor train-p :initarg :train-p :initform nil)
   (postcomp-p :accessor postcomp-p :initarg :postcomp-p :initform nil)
   (trial-num :accessor trial-num :initarg :trial-num :initform nil)
   ))


(defmethod write-trial ((trl x84-trial) strm)
  (terpri strm)
  (tab-output (list
               (xcond trl)
               (trial-kind trl)
               (trial-block trl)
               (rt trl)
               (nerrors trl)
               (if (train-p trl) 1 0)
               (if (postcomp-p trl) 1 0)
               (trial-num trl))
              strm))




(defclass x84-training-block (trial-block)
  ())



;;;; ---------------------------------------------------------------------- ;;;;
;;;; the main control window itself


(defclass main-control-window (event-exp-window)
  ((day :accessor day :initarg :day :initform 1)
   (base-path :accessor base-path :initarg :base-path 
              :initform (mac-directory-namestring "dissertation-model:model-data;"))
   (postcomp-p :accessor postcomp-p :initarg :postcomp-p :initform nil)
   (ncomp :accessor ncomp :initarg :ncomp :initform nil)
   (trial-num :accessor trial-num :initarg :trial-num :initform 0)
   (current-btn :accessor current-btn :initarg :current-btn :initform :PHASER)
   (train-p :accessor train-p :initarg :train-p :initform nil)
   (action-log :accessor action-log :initarg :action-log :initform nil)
   (pswitch-count :accessor pswitch-count :initarg :pswitch-count :initform 0)
   (tswitch-count :accessor tswitch-count :initarg :tswitch-count :initform 0)
   (pswitch :accessor pswitch :initarg :pswitch :initform nil)
   (tswitch :accessor tswitch :initarg :tswitch :initform nil)
   (jswitch-count :accessor jswitch-count :initarg :jswitch-count :initform 0)
   (jswitch :accessor jswitch :initarg :jswitch :initform nil)
   (trials-completed :accessor trials-completed :initarg :trials-completed :initform 0)
   (transporter-window-orders :accessor transporter-window-orders :initarg :transporter-window-orders :initform nil)
   (jammer-window-same-orders :accessor jammer-window-same-orders :initarg :jammer-window-same-orders :initform nil)
   (jammer-window-different-orders :accessor jammer-window-different-orders :initarg :jammer-window-different-orders :initform nil)
   (last-update :accessor last-update :initarg :last-update :initform 0)
;;   (training-p :accessor training-p :initarg :training-p :initform nil)
   (task-wind :accessor task-wind :initarg :task-wind :initform nil)
   (single-kind :accessor single-kind :initarg :single-kind :initform nil)
   (training-msgs 
    :accessor training-msgs :initarg :training-msgs 
    :initform 
    (cond
     ((= *exp-cond* 3)
      '((:phaser ((:POWER-connected "click the Smokey button.")
                  (:CHARGE "click the Entomologist button.")
                  (:STOP-CHARGING "click the Filmmaker button.")
                  (:SETTINGS "click the Solenoid button.")
                  (:FOCUS-SET "click the Reflecting button.")
                  (:FIRING "click the Redcoats button.")
                  (:TRACKING "click the Headings button.")
                  (:MAIN-CONTROL "click the Drafty button.")
                  (:SHOOT "press the space bar to fire the phaser.")
                  (:SLIDER "set position of the Solenoid-Relecting slider.")))
        (:TRANSPORTER ((:SCANNER-ON "click the Scanner On button.")
                       (:ACTIVE-SCAN "click the Active Scan button.")
                       (:LOCK-SIGNAL "click the Lock Signal button.")
                       (:SCANNER-OFF "click the Scanner Off button.")
                       (:ENTER-FREQUENCY "click the Enter Frequency button.")
                       (:type "type a number into the field.")
                       (:ACCEPT-FREQUENCY "click the Accept Frequency button.")
                       (:TRANSPORTER-POWER "click the Transporter Power button.")
                       (:SYNCHRONOUS-MODE "click the Synchronous Mode button.")
                       (:SHOOT "click the moving target.")
                       (:MAIN-CONTROL "click the Main Control button.")))
#|
        (:NAVIGATION ((:CONFIRM-COURSE "click the Confirm Course button.")
                      (:type "type in course changes.")
                      (:ACCEPT-COURSE "click the Accept Course button.")))
|#
        (:JAMMER ((:SCANNER-ON "click the Tracker On button.")
                  (:ACTIVE-SCAN "click the Modulator button.")
                  (:LOCK-SIGNAL "click the Fix Frequency button.")
                  (:SCANNER-OFF "click the Tracker Off button.")
                  (:ENTER-FREQUENCY "click the Calibration button.")
                  (:type "type a number into the calibration field.")
                  (:ACCEPT-FREQUENCY "click the Accept Calibration button.")
                  (:TRANSPORTER-POWER "click the Auxiliary Power button.")
                  (:SYNCHRONOUS-MODE "click the Desynchronize button.")
                  (:SHOOT "click the moving target.")
                  (:MAIN-CONTROL "click the Main Control button.")))))
     (t '((:PHASER ((:POWER-connected "click the Electrical button.")
                    (:CHARGE "click the Generator button.")
                    (:STOP-CHARGING "click the Kilowatts button.")
                    (:SETTINGS "click the Lens button.")
                    (:FOCUS-SET "click the Focus button.")
                    (:FIRING "click the Cannon button.")
                    (:TRACKING "click the Shot button.")
                    (:MAIN-CONTROL "click the Main button.")
                    (:SHOOT "press the space bar to fire the phaser.")
                    (:SLIDER "set position of the lens focus slider.")))
          (:TRANSPORTER ((:SCANNER-ON "click the Scanner On button.")
                         (:ACTIVE-SCAN "click the Active Scan button.")
                         (:LOCK-SIGNAL "click the Lock Signal button.")
                         (:SCANNER-OFF "click the Scanner Off button.")
                         (:ENTER-FREQUENCY "click the Enter Frequency button.")
                         (:type "type a number into the field.")
                         (:ACCEPT-FREQUENCY "click the Accept Frequency button.")
                         (:TRANSPORTER-POWER "click the Transporter Power button.")
                         (:SYNCHRONOUS-MODE "click the Synchronous Mode button.")
                         (:SHOOT "click the moving target.")
                         (:MAIN-CONTROL "click the Main Control button.")))
#|
          (:NAVIGATION ((:CONFIRM-COURSE "click the Confirm Course button.")
                        (:type "type in course changes.")
                        (:ACCEPT-COURSE "click the Accept Course button.")))
|#
          (:JAMMER ((:SCANNER-ON "click the Scanner On button.")
                    (:ACTIVE-SCAN "click the Active Scan button.")
                    (:LOCK-SIGNAL "click the Lock Signal button.")
                    (:SCANNER-OFF "click the Scanner Off button.")
                    (:ENTER-FREQUENCY "click the Enter Frequency button.")
                    (:type "type a number into the frequency field.")
                    (:ACCEPT-FREQUENCY "click the Accept Frequency button.")
                    (:TRANSPORTER-POWER "click the Transporter Power button.")
                    (:SYNCHRONOUS-MODE "click the Synchronous Mode button.")
                    (:SHOOT "click the moving target.")
                    (:MAIN-CONTROL "click the Main Control button.")))))
     )
    )
   )
  (:default-initargs 
    :close-box-p nil
    :xnum 84
    :back-color 6142666
    :view-position #@(50 50)
    :VIEW-SUBVIEWS
    (list (make-dialog-item
           'static-text-dialog-item
           #@(400 166)
           #@(235 28)
           "Main Control"
           'nil
           :view-font '("Helvetica" 36 :srcor :bold (:color-index 0)))
          (make-dialog-item
           'static-text-dialog-item
           #@(276 240)
           #@(481 150)
           ""
           'nil
           :view-nick-name :feedback
           :view-font '("Avant Garde" 18 :srcor :plain (:color-index 0)))
          (make-dialog-item
           'button-dialog-item
           #@(209 431)
           #@(135 45)
           "Phaser"
           #'(lambda (self) 
               (do-button (view-window self) (view-nick-name self)))
           :view-nick-name :phaser
           :view-font
           '("Charcoal" 18 :srcor :plain (:color-index 0))
           :default-button nil)
          (make-dialog-item
           'button-dialog-item
           #@(521 431)
           #@(135 45)
           "Transporter"
           #'(lambda (self) 
               (do-button (view-window self) (view-nick-name self)))
           :view-nick-name :TRANSPORTER
           :view-font
           '("Charcoal" 18 :srcor :plain (:color-index 0))
           :default-button nil)
          (make-dialog-item
           'button-dialog-item
           #@(677 431)
           #@(150 45)
           "Jammer"
           #'(lambda (self) 
               (do-button (view-window self) (view-nick-name self)))
           :view-nick-name :jammer
           :view-font
           '("Charcoal" 18 :srcor :plain (:color-index 0))
           :default-button nil)
          )
    ))


(defun auto-read-from-file  (cond)
  "Reads from a file in a hard-coded location, using a hard-coded filename, and chooses the version appropriate to the given condition number."
  (with-open-file (str 
                   (concatenate 'string "Orca:Users:frank:School:Dissertation:ACT-R Model:X84-"
                                (format nil "~A" cond)
                                " instructions.txt") :direction :input)
      (read str)))

(defmethod initialize-instance :after ((wind main-control-window) &key day snum cond)
;;  (when *actr-enabled-p* (window-hide wind))
  (when snum (setf (snum wind) snum))
  (if day
    (setf (day wind) day)
    (setf (day wind) (get-num-from-user 1 2 "Day number?")))       ; check
  (setf (xcond wind) cond)
  (setf *exp-cond* (xcond wind))
  ;; get instructions for those conditions which need it
#|
  (unless *actr-enabled-p* 
    (if cond
      (setf (instructions wind) (auto-read-from-file cond))
      (when  (eq (day wind) 2)
        (warning "Please locate the instructions.")
        (setf (instructions wind) (read-from-chosen-file)))))
|#
  (setf (train-p wind) (= 1 (day wind)))
  (when (train-p wind)
    (setf (url wind) nil)
    (setf (transporter-window-orders wind) 
          (randomize-list 
           '(TRANSPORTER-WINDOW-ORDER0 
             TRANSPORTER-WINDOW-ORDER0 
             TRANSPORTER-WINDOW-ORDER0
             TRANSPORTER-WINDOW-ORDER1
             TRANSPORTER-WINDOW-ORDER1 
             TRANSPORTER-WINDOW-ORDER1)))
    (setf (jammer-window-same-orders wind) 
          (randomize-list 
           '(jammer-window-same-ORDER0
             jammer-window-same-ORDER0 
             jammer-window-same-ORDER0 
             jammer-window-same-ORDER1 
             jammer-window-same-ORDER1 
             jammer-window-same-ORDER1)))
    (setf (jammer-window-different-orders wind) 
          (randomize-list 
           '(jammer-window-different-ORDER0 
             jammer-window-different-ORDER0 
             jammer-window-different-ORDER0 
             jammer-window-different-ORDER1 
             jammer-window-different-ORDER1 
             jammer-window-different-ORDER1))))
  (setf (nblocks wind) (if (train-p wind) 4 1)) 
  
  ;; This is where n of each trial type is set: phaser, transporter, nav jammer.
  (setf (block-lst wind) (do-stimgen (train-p wind) 12 12 0 12))

  ;; for testing
;;  (setf (block-lst wind) (do-stimgen (train-p wind) 2 2 0 2))


  
  (unless *actr-enabled-p*
    (with-open-file (strm (make-data-path wind "dat") :direction :output
                          :if-exists :append :if-does-not-exist :create)
      (format strm "COND	KIND	BLOCK	RT	NERR	TRAIN?	PCOMP?	TNUM")))
  (setf (postcomp-p wind) t)	
;;  (when  (eq (day wind) 2) 
;;    (show-instructions wind))
  (when *wmt* (reset-time *wmt*))
  (start-timing (timer wind))
;;  (when *actr-enabled-p* (goal-focus do-main-control))
  )


(defmethod task-window ((wind main-control-window))
  (feedback wind "")
  (case (current-btn wind)
    (:phaser (prog1
               (setf (task-wind *experiment*) (make-instance 
                                                (case (xcond wind)
                                                  (0 'phaser-window-interruption)
                                                  (1 (if (pswitch wind)
                                                       'phaser-window-interruption
                                                       'phaser-window-no-interruption))
                                                  (2 'phaser-window-no-interruption)
                                                  (3 'phaser-window-semantic-control))
                                                :postcomp-p (postcomp-p wind)
                                                :snd-player *sp*
                                                :pcondition (xcond wind)
                                                :training-p (train-p wind)
                                                :out-path (protocol-fpath wind)))
               (goal-focus phaser-goal)
               (set-buffer-chunk 'imaginal 'phaser-imaginal)))
    
    
    (:transporter (prog1
                    (setf (task-wind *experiment*) (make-instance 'transporter-window-static-subtask-order
                                                     :training-p (train-p wind)
                                                     :snd-player *sp*
                                                     :tcondition (xcond wind)
                                                     :out-path (protocol-fpath wind)))
                          (goal-focus transporter-goal)
                          (set-buffer-chunk 'imaginal 'transporter-imaginal)))
    
    
    (:jammer (prog1
               (setf (task-wind *experiment*) (make-instance 
                 (case (xcond wind)
                   (0 'jammer-window-0)
                   (1 'jammer-window-1)
                   (2 (if (jswitch wind)
                          'jammer-window-0
                          'jammer-window-1))
                   (3 'jammer-window-static-subtask-order))
                 :training-p (train-p wind)
                 :snd-player *sp*
                 :out-path (protocol-fpath wind)))
               (goal-focus jammer-goal)
               (set-buffer-chunk 'imaginal 'jammer-imaginal))))
  (when *actr-enabled-p* (trigger-reward *correct-reward*))
  (when *actr-enabled-p* (install-device (task-wind *experiment*)))
;;  (terpri)
;;  (format t "device installed: ~A~%" (task-wind *experiment*))
  (when *actr-enabled-p* (proc-display :clear t)))






(defun get-num-from-user (min max &optional prompt)
  (let ((num nil))
    (unless prompt (setf prompt 
                         (format nil "Please enter a number between ~D and ~D"
                                 min max)))
    (while (not (and (numberp num) (>= num min) (<= num max)))
      (setf num (read-from-string 
                 (get-string-from-user prompt 
                                       :allow-empty-strings nil))))
    num))



(defmethod feedback ((wind main-control-window) (msg string))
  (set-dialog-item-text (view-named :FEEDBACK wind) msg))


(defmethod append-feedback ((wind main-control-window) (msg string))
  (let ((fv (view-named :FEEDBACK wind)))
    (set-dialog-item-text fv (mkstr (dialog-item-text fv) #\return msg))))



(defmethod do-button ((wind main-control-window) btn)
  (if (neq btn (current-btn wind))
    (progn
      (when *actr-enabled-p* (trigger-reward *error-reward*))
      (when *actr-enabled-p* (new-other-sound 'wrong-click-buzzer 1.0 0 0 (mp-time) 'external 'wrong-click-buzzer)))
    (let ((kind (trial-kind (current-trial wind))))
      (when (and (eq (day wind) 2)
                 (or 
                  (and (pswitch wind) (= kind 0))
                  (and (jswitch wind) (= kind 3)))
                 (null (instructions wind)))
        (instruct-actr wind)
        )
      (multiple-value-bind (nerr time p-act)
                           (task-window wind)
;;        (when (= (trial-num wind) 36) (finish-block wind (car (block-lst wind))))
        (incf (trial-num wind))
        (fill-trial wind nerr time)
        (erase-rect wind #@(0 0) #@(1024 768))
        (view-draw-contents wind)
        (cond ((not (train-p wind))
               ;; (feedback wind
                  ;;       (format nil "You took ~S seconds and made ~S error(s)."
                    ;;             time nerr))
;;               (bonus-points? wind time)
               ;; (finish-trial wind)
               )
              (t
               (if (zerop nerr)
                 (feedback wind "")
                 (feedback wind (error-string wind p-act)))
               (when (zerop nerr) (incf (ncomp wind))) 
               (append-feedback 
                wind 
                (format nil "You have completed ~S trial(s) successfully."
                        (ncomp wind)))
               (when (and (zerop nerr) (= (ncomp wind) 1))
                 (warning "Please return the manual to the experimenter."))
               (if 
                 (or
                  (and (< (xcond wind) 3) (or (= kind 1) (= kind 3)) (= (ncomp wind) 6))
                  (and (or (= kind 0) (= kind 2) 
                           (and (= 3 (xcond wind)) (or (= kind 1) (= kind 3)))) (= (ncomp wind) 4)))
                 (finish-training-task wind)
                 (continue-training-task wind))
               (when (zerop nerr)
;;                  (format t "trial kind is: ~a~%" kind)
                 (when (= kind 0) (incf (pswitch-count wind)))
                 (when (= kind 1) (incf (tswitch-count wind)))
                 (when (= kind 3) (incf (jswitch-count wind)))
                 (incf (trials-completed *experiment*))
;;                 (format t "pswitch-count: ~a~%" (pswitch-count wind))
;;                 (format t "tswitch-count: ~a~%" (tswitch-count wind))
;;                 (format t "jswitch-count: ~a~%" (jswitch-count wind))
                 )
               ))
        ;(format t "switch count: ~s" (switch-count wind))
        ))))

(defmethod instruct-actr ((wind main-control-window))
  (let ((kind (trial-kind (current-trial wind))))
    (when (= kind 0)
      (add-dm
       (changed-phaser-imaginal isa problem-state interface phaser top do-changed-phaser middle charge-phaser bottom power-connected)

       (intervene-stop-charging isa change-instruction current-flag stop-charging do-step settings do-subtask set-focus next-flag firing)
       (intervene-firing isa change-instruction current-flag firing do-step stop-charging do-subtask charge-phaser next-flag settings)
       (intervene-settings isa change-instruction current-flag settings do-step firing do-subtask fire-phaser next-flag stop-charging)
       )
      (mod-chunk phaser-imaginal top do-changed-phaser)
      (mod-chunk phaser-goal state-micro stop-charging)

      (set-similarities (power-connected2 settings -1))

      (add-sji
       (stop-charging intervene-stop-charging 2.5) (intervene-stop-charging stop-charging 2.5)
       (firing intervene-firing 2.5) (intervene-firing firing 2.5)
       (settings intervene-settings 2.5) (intervene-settings settings 2.5)

       (stop-charging intervene-firing -0.5) (intervene-firing stop-charging -0.5)
       (stop-charging intervene-settings -0.5) (intervene-settings stop-charging -0.5)
       (firing intervene-stop-charging -0.5) (intervene-stop-charging firing -0.5)
       (firing intervene-settings -0.5) (intervene-settings firing -0.5)
       (settings intervene-stop-charging -0.5) (intervene-stop-charging settings -0.5)
       (settings intervene-firing -0.5) (intervene-firing settings -0.5)
       )

      )

    (when (= kind 3)
      (add-dm
       (changed-jammer-imaginal isa problem-state interface transporter top do-changed-jammer middle scan bottom active-scan)
       
       (intervene-jammer-scanner-on isa change-instruction current-flag scanner-on do-step active-scan do-subtask scan next-flag lock-signal)
       (intervene-jammer-lock-signal isa change-instruction current-flag lock-signal do-step scanner-off do-subtask scan next-flag enter-frequency)
       (intervene-jammer-enter-frequency isa change-instruction current-flag enter-frequency do-step lock-signal do-subtask scan next-flag scanner-off)
       (intervene-jammer-scanner-off isa change-instruction current-flag scanner-off do-step scanner-on do-subtask scan next-flag active-scan)
       (intervene-jammer-active-scan isa change-instruction current-flag active-scan do-step enter-frequency do-subtask frequency next-flag scanner-on)
       )

      

      (add-sji
       (intervene-jammer-scanner-on scanner-on 1) (scanner-on intervene-jammer-scanner-on 1)
       (intervene-jammer-lock-signal lock-signal 2) (lock-signal intervene-jammer-lock-signal 2)
       (intervene-jammer-enter-frequency enter-frequency 1.3) (enter-frequency intervene-jammer-enter-frequency 1.3)
       (intervene-jammer-scanner-off scanner-off 2) (scanner-off intervene-jammer-scanner-off 2)
       (intervene-jammer-active-scan active-scan 1.6) (active-scan intervene-jammer-active-scan 1.6)
       
       (intervene-jammer-scanner-on lock-signal 0.2) (lock-signal intervene-jammer-scanner-on 0.2)
       (intervene-jammer-scanner-on enter-frequency 0.5) (enter-frequency intervene-jammer-scanner-on 0.5)
       (intervene-jammer-scanner-on scanner-off 0.2) (scanner-off intervene-jammer-scanner-on 0.2)
       (intervene-jammer-scanner-on active-scan 0.4) (active-scan intervene-jammer-scanner-on 0.4)
       (intervene-jammer-lock-signal scanner-on 0.6) (scanner-on intervene-jammer-lock-signal 0.6)
       (intervene-jammer-lock-signal enter-frequency 0.5) (enter-frequency intervene-jammer-lock-signal 0.5)
       (intervene-jammer-lock-signal scanner-off 0.2) (scanner-off intervene-jammer-lock-signal 0.2)
       (intervene-jammer-lock-signal active-scan 0.4) (active-scan intervene-jammer-lock-signal 0.4)
       (intervene-jammer-enter-frequency scanner-on 0.6) (scanner-on intervene-jammer-enter-frequency 0.6)
       (intervene-jammer-enter-frequency lock-signal 0.2) (lock-signal intervene-jammer-enter-frequency 0.2)
       (intervene-jammer-enter-frequency scanner-off 0.2) (scanner-off intervene-jammer-enter-frequency 0.2)
       (intervene-jammer-enter-frequency active-scan 0.4) (active-scan intervene-jammer-enter-frequency 0.4)
       (intervene-jammer-scanner-off scanner-on 0.6) (scanner-on intervene-jammer-scanner-off 0.6)
       (intervene-jammer-scanner-off lock-signal 0.2) (lock-signal intervene-jammer-scanner-off 0.2)
       (intervene-jammer-scanner-off enter-frequency 0.5) (enter-frequency intervene-jammer-scanner-off 0.5)
       (intervene-jammer-scanner-off active-scan 0.4) (active-scan intervene-jammer-scanner-off 0.4)
       (intervene-jammer-active-scan scanner-on 0.2) (scanner-on intervene-jammer-active-scan 0.6)
       (intervene-jammer-active-scan lock-signal 0.2) (lock-signal intervene-jammer-active-scan 0.2)
       (intervene-jammer-active-scan enter-frequency 0.5) (enter-frequency intervene-jammer-active-scan 0.5)
       (intervene-jammer-active-scan scanner-off 0.2) (scanner-off intervene-jammer-active-scan 0.2)
       ) ;; add-aji closing parenthesis


      (mod-chunk jammer-imaginal top do-changed-jammer)
      (mod-chunk jammer-goal state-micro scanner-on))

    (setf (instructions wind) t)
    ))
             




(defmethod finish-trial :before ((wind main-control-window))
  (unless (single-kind wind)
    (let ((kind (trial-kind (current-trial wind))))
      (when (= kind 0) (incf (pswitch-count wind)))
      (setf (pswitch wind) (and (= 1 (xcond wind)) (>= (pswitch-count wind) (/ 12 2))))
      (when (= kind 1) (incf (tswitch-count wind)))
      ;;    (setf (tswitch wind) (>= (tswitch-count wind) (/ 12 2)))
      (when (= kind 3) (incf (jswitch-count wind)))
      (setf (jswitch wind) (and (= 2 (xcond wind)) (>= (jswitch-count wind) (/ 12 2))))
      ;;  (format t "(size (car (block-lst wind))): ~A~%" (size (car (block-lst wind))))
      ;; (format t "pswitch-count: ~a~%" (pswitch-count wind))
      ;;  (format t "pswitch: ~A~%" (pswitch wind))
      ;; (format t "tswitch-count: ~a~%" (tswitch-count wind))
      ;;  (format t "~A~%" (tswitch wind))
      ;; (format t "jswitch-count: ~a~%" (jswitch-count wind))
      ))
    (incf (trials-completed *experiment*))
    )





(defmethod bonus-points? ((wind main-control-window) (time number))
  ;; <time> comes in seconds, so this is easy
  (let ((bonus
         (case (current-btn wind)
           (:phaser
            (cond ((< time 20) 100)
                  ((< time 40) (- 80 time))
                  (t 0)))
           (:TRANSPORTER
            (cond ((< time 13) 100)
                  ((< time 22) (- 80 time))
                  (t 0)))
           (:jammer
            (cond ((< time 13) 100)
                  ((< time 22) (- 80 time))
                  (t 0)))
           (:navigation
            (cond ((< time 10) 100)
                  ((< time 20) (- 80 time))
                  (t 0))))))
    (when (not (zerop bonus))
      (append-feedback 
       wind 
       (format nil "Your time earned you a bonus of ~S points." bonus))
      (delta-score *sw* bonus))))




(defmethod fill-trial ((wind main-control-window) nerr time)
  (let ((ct (current-trial wind)))
    (setf (rt ct) time
          (nerrors ct) nerr
          (train-p ct) (train-p wind)
          (xcond ct) (xcond wind)
          (postcomp-p ct) (when (eq (current-btn wind) :PHASER)
                            (postcomp-p wind))
          (trial-num ct) (trial-num wind))))


(defmethod error-string ((wind main-control-window) (p-act proc-action))
  (let ((msg-alst (second (assoc (current-btn wind) (training-msgs wind)))))
    (mkstr
     "You must first "
     (second (assoc (expected p-act) msg-alst)))))




(defmethod continue-training-task ((wind main-control-window))
  (push (current-trial wind) (trial-lst (current-block wind)))
  (setup-trial wind (set-new-training-trial wind)))


(defmethod set-new-training-trial ((wind main-control-window))
  ;;  (format t "Does this run every time a training trial runs?")
  (setf (current-trial wind)
        (make-instance 'X84-trial
          :trial-kind (kind (current-block wind))
          :train-p t)))


(defmethod finish-training-task ((wind main-control-window))
  (feedback wind 
            (mkstr "Congratulations, you have finished training with the "
                   (current-btn wind)
                   "."))
  (finish-block wind (current-block wind)))


(defmethod window-null-event-handler ((wind main-control-window))
  (unless (or *actr-enabled-p* (train-p wind)) (do-wm-check *wmt* wind)))


;;; experiment window menthods

(defmethod setup-experiment ((wind main-control-window))
  ;(setf *tracking-p* nil)
  (when (train-p wind)
    (let ((train-items (mapcar #'(lambda (b) (knum-key (kind b)))
                                 (block-lst wind))))
      (warning (format nil "The training order will be: ~%~A~%~A~%~A~%~A"
                       (first train-items) (second train-items)
                       (third train-items) (fourth train-items)))
      :size #@(400 300))))



(defmethod run-block ((wind main-control-window) (blk x84-training-block))
  (setf (trial-num wind) 0)
  (setf (ncomp wind) 0)
  (setup-trial wind (set-new-training-trial wind)))



(defmethod finish-block :before ((wind main-control-window)
                                   (blk x84-training-block))
  (setf (trial-lst blk) (nreverse (trial-lst blk))))


(defmethod finish-block ((wind main-control-window) (blk trial-block))
  (write-block wind blk)
  (gc)
  (incf (cblock wind))
  )


(defmethod setup-trial :after ((wind main-control-window) (trl trial))
  (let ((keywd (knum-key (trial-kind trl))))
    (setf (current-btn wind) keywd)
    (if (train-p wind)
      (append-feedback wind
                       (format nil "Please train with the ~A." (mkstr keywd)))
        (append-feedback 
         wind (format nil "~A" (mkstr keywd)))
        )
    (do-button wind (current-btn wind)))
;;  (advance-state wind)
  )



(defmethod finish-experiment :before ((wind main-control-window))
  ;; drop a new chunk into the goal buffer to indicate that the model is now done with the experiment run
  (unless (or *actr-enabled-p* (train-p wind))
    ;; write the WM data
    (with-open-file (strm (wm-fpath wind) :direction :output
                          :if-exists :append :if-does-not-exist :create)
      (print-results *wmt* strm))
    (set-mac-file-creator (wm-fpath wind) :|R*ch|)
    (lock-file (wm-fpath wind)))
  (set-mac-file-creator (protocol-fpath wind) :|R*ch|)
;;  (lock-file (protocol-fpath wind))
  (unless *actr-enabled-p* 
    (when (find-window "Score") 
      (let ((final-score (score *sw*)))
        (window-close *sw*)
        ;;      (format t "~%Total score was: ~S" final-score)
        ;; write the final score
        (with-open-file (strm (make-data-path wind "txt") :direction :output 
                              :if-exists :append :if-does-not-exist :create)
          (format strm "~%Final score:  ~S" final-score))
        (setf *sw* nil)))))


(defmethod finish-experiment ((wind main-control-window))
  (beep)
  (terpri)
  (format t "The experiment has finished.~%")
;;  (format t "cblock = ~A~%" (cblock wind))
;;  (format t "trials left = ~A~%" (trials-left))
;;  (format t "current idx = ~A~%" (current-idx (car (block-lst *experiment*))))
  (when *actr-enabled-p* 
    (progn
      (goal-focus finished-experiment)
      (proc-display :clear t)
      (schedule-break-relative 0 :details "Stop the system now")))
  (window-close *sw*)
  (setf *sw* nil)
  (window-close wind)
  (window-close (task-wind wind))
  (#_showcursor)
  (unless *actr-enabled-p*
    (when (and (base-path wind) (member (write-type wind) '(:SS :BOTH)))
      (let ((path (make-data-path wind "txt")))
        (set-mac-file-creator path :|R*ch|)
        (lock-file path)))
    (when (member (write-type wind) '(:LISP :BOTH))
      (with-open-file (strm (make-data-path wind "lisp") :direction :output 
                            :if-exists :supersede :if-does-not-exist :create)
        (format strm ")"))))
  ;;  (menubar-show)
  )

#|
(defmethod finish-experiment ((wind main-control-window))
  (beep)
  (window-close wind)
  (#_showcursor)
  (menubar-show)
  )
|#


(defmethod write-block ((wind main-control-window) (blk trial-block))
  (unless *actr-enabled-p*
    (if (null (base-path wind))
      ;; if not writing to a file, write SS data to terminal
      (dolist (the-trial (trial-lst blk))
        (write-trial the-trial t))
      ;; otherwise, need to know what type to write
      (progn
        (when (member (write-type wind) '(:SS :BOTH))
          (with-open-file (strm (make-data-path wind "txt") :direction :output 
                                :if-exists :append :if-does-not-exist :create)
            (dolist (the-trial (trial-lst blk))
              (write-trial the-trial strm))))
        (when (member (write-type wind) '(:LISP :BOTH))
          (with-open-file (strm (make-data-path wind "lisp") :direction :output 
                                :if-exists :append :if-does-not-exist :create)
            (write-readable (trial-lst blk) strm)))))))


;;; utils

(defmethod protocol-fpath ((wind main-control-window))
  (make-pathname :directory (base-path wind)
                 :name (format nil "s~3,'0Dd~Dprot" (snum wind) (day wind))
                 :type "dat"))


(defmethod wm-fpath ((wind main-control-window))
  (make-pathname :directory (base-path wind)
                 :name (format nil "s~3,'0Dd~Dwm" (snum wind) (day wind))
                 :type "dat"))


(defmethod make-data-path ((wind main-control-window) type)
  (make-pathname :directory (base-path wind)
                 :name (format nil "s~3,'0Dd~Dtrial" (snum wind) (day wind))
                 :type type))


(defmethod current-block ((wind main-control-window))
  (nth (cblock wind) (block-lst wind)))

(defmethod truekey ((wind main-control-window))
  (let ((keywd (current-btn wind)))
    (when (eq keywd :transporter)
      (setf keywd (if (zerop (xcond wind)) :transporter0 :transporter1)))
    keywd))


;; avoid use of modal-dialogs because ACT-R doesn't like them
#|
(defmethod finish-task ((wind procedure-window))
  (unless (training-p wind)
    (delta-score *sw* *step-value))
  (write-log wind)
  (prog1
    (values (num-errors wind) (elapsed-secs wind) (first (action-log wind)))
    (window-close wind)))

(defmethod advance-state ((wind procedure-window))
  (setf (advance-p wind) t)
  (gui-check wind)                      ; r2 new
  (check-state-update wind (curr-state wind))
  (when (and (advance-p wind) (< (state-num wind) 12))
    (incf (state-num wind)))
  (unless (training-p wind)
    (unless (eq (last-correct *sw*) (curr-state wind))
      (delta-score *sw* *step-value))
    (setf (last-correct *sw*) (curr-state wind))))

(defmethod curr-state ((wind procedure-window))
  (let ((sn (state-num wind)))
    (when (> sn 11) (setf sn 11))
    (svref (state-vec wind) (state-num wind))))
|#




;;; stimulus generation stuff


(defun do-stimgen 
       (train-p n-phaser-trials n-transporter-trials n-nav-trials n-jammer-trials &key (single-kind nil))
  "version without dummy nav trial as one extra"
  ;; &optional (ntrials *ntrials*))
  (if train-p
    (randomize-list
     (list (make-instance 'X84-training-block :kind 0)
           (make-instance 'X84-training-block :kind 1)
           (make-instance 'x84-training-block :kind 2)          
           (make-instance 'X84-training-block :kind 3)
           ))
    (list 
     (let ((stim-lst (build-basic-trials 
                        n-phaser-trials 
                        n-transporter-trials 
                        n-nav-trials 
                        n-jammer-trials))) 
       (make-instance 'trial-block
         :size  (length stim-lst)
         :trial-lst (if single-kind
                      stim-lst
                      (order-stim-list stim-lst))
         )))))

#|
(defun do-stimgen 
       (train-p n-phaser-trials n-transporter-trials n-nav-trials n-jammer-trials)
  "version with dummy nav trial as one extra"
  ;; &optional (ntrials *ntrials*))
  (if train-p
    (randomize-list
     (list (make-instance 'X84-training-block :kind 0)
           (make-instance 'X84-training-block :kind 1)
           (make-instance 'x84-training-block :kind 2)          
           (make-instance 'X84-training-block :kind 3)
           ))
    (list 
     (let ((stim-lst (build-basic-trials 
                        n-phaser-trials 
                        n-transporter-trials 
                        n-nav-trials 
                        n-jammer-trials))) 
       (make-instance 'trial-block
         :size  (1+ (length stim-lst))
         :trial-lst
         (add-dummy-trial (order-stim-list stim-lst))
         )))))
|#

(defun make-trials (num &rest args)
  (let (accum)
    (dotimes (i num)
      (push (apply #'make-instance 'X84-trial args) accum))
    accum))


(defun build-basic-trials 
       (n-phaser-trials n-transporter-trials n-nav-trials n-jammer-trials)
  (append 
   (make-trials n-phaser-trials :trial-kind 0)       ; phaser
   (make-trials n-transporter-trials :trial-kind 1)       ; transporter
   (make-trials n-nav-trials :trial-kind 2)       ; navigation             
   (make-trials n-jammer-trials :trial-kind 3)	; jammer
   ))



(defun order-stim-list (in-lst &optional (tries 20))
  (let ((lst (randomize-list in-lst)))
    (dotimes (i tries)
      (when (stim-sequence-ok lst)
        (return-from order-stim-list lst))
      (setf lst (randomize-list lst)))
    (format t "~% Please report 'randomization failure' to Mike.")
    lst))

(defun add-dummy-trial (lst)
  (nreverse (append (make-trials 1 :trial-kind 2) (nreverse lst))))

(defun stim-sequence-ok (lst)
  (< (maxrun lst #'trial-kind) 4))

(defun maxrun (lst &optional key)
  (apply #'max 
         (mapcar #'first 
                 (remove-if #'atom
                            (compress 
                             (if key
                               (mapcar key lst)
                               lst))))))

(defun compress (x)
  (if (consp x) 
    (compr (car x) 1 (cdr x))
    x))

(defun compr (elt n lst)
  (if (null lst)
    (list (n-elts elt n))
    (let ((next (car lst)))
      (if (eql next elt)
        (compr elt (+ n 1) (cdr lst))
        (cons (n-elts elt n)
              (compr next 1 (cdr lst)))))))

(defun n-elts (elt n)t
       (if (> n 1)
         (list n elt)
         elt))


;;; Re-coded WM-tester sound stuff so that ACT-R may hear

(defmethod new-probe ((wmt wm-tester))
  (let ((probe (random-item (snd-lst wmt))))
    (push probe (curr-stims wmt))
    (if *actr-enabled-p*
      (new-word-sound probe)
      (play-snd (snd-player wmt) probe t))
    (setf (last-time wmt) (get-internal-real-time))))

(defmethod do-probe ((wmt wm-tester) (wind window))
  (let ((start (get-internal-real-time))
        (finish nil)
        (response nil) 
        (probe "")
        (correct-p nil))
    (push (make-instance 'proc-action
            :latency  (round (start-stop-timer (timer wind)))
            :step-num -1 :expected :probe :got :probe)
          (action-log wind))
    (setf response (qkstr-dlg wind (probe-depth wmt)))
    (setf finish (get-internal-real-time))
    (awhen (find-window "Recall" 'QKSTR-DIALOG)
           (window-close it))
    (without-interrupts
     (setf response (string-downcase response))
     (dolist (item (nreverse (curr-stims wmt)))
       (setf probe (concatenate 'string probe item)))
     (when (and (not (equal probe ""))
                (stringp response) (= (length response) 3))
       (if (string= response 
                    (subseq probe (- (length probe) (probe-depth wmt))))
         (setf correct-p 1)
         (setf correct-p 0))
       (when (= correct-p 0)
         (if *actr-enabled-p*
           (new-other-sound 'wm-warning-tone 1.0 0 0 (mp-time) 'external 'wm-warning-tone)
           (play-snd (snd-player wmt) "warning tone"))
         (delta-score *sw* *wm-penalty)
         ))
     (push (list correct-p probe response finish start) (results wmt))
     (reset-probe wmt 1000)
     ;(values (- finish start) correct-p)
     )))

(defun qkstr-dlg (wind &optional (strlen 3))
  (if *actr-enabled-p*
    (new-tone-sound 880 0.5)
    (play-snd *sp* "Indigo"))
  (spin-for (timer *experiment*) 150)
  (event-dispatch)
  (modal-dialog (make-instance 'qkstr-dialog :strlen strlen
                               :wind wind) t))





;;;; ---------------------------------------------------------------------- ;;;;
;;;;

(defun begin-experiment (&key cond (subj 0) (day 2) phaser-speed transporter-speed)
;;  (setf *run-time* run-time)
  (setf *random-state* (make-random-state t))
  (when (numberp phaser-speed) (setf *phaser-speed* phaser-speed))
  (when (numberp transporter-speed) 
    (setf *transporter-speed* transporter-speed))
;;  (menubar-hide)
  (unless *actr-enabled-p*
    (setf *sp* 
          (make-instance 
            'snd-player :res-file nil
            :preload '("Warning" "Whit" "Phaser" "Torpedo" "Whip" "damn" "expolsion"
                       "Transporter TNG" "Probe" "a" "b" "c" "d" "e" "f" "g" "h" 
                       "i" "j" "k" "l" "m" "n" "o" "p" "q" "r" "s" "t" "u" "v" 
                       "w" "x" "y" "z" "uh oh" "glass"))))
  (when (and *sp* *actr-enabled-p*) (setf *sp* nil))
  (when *experiment*
;;    (when (eq 'main-control-window (class-name (class-of *experiment*)))
    (when (task-wind *experiment*)
      (window-close (task-wind *experiment*)))
      (window-close *experiment*))
  (if cond
    (setf *exp-cond* cond)
    (setf *exp-cond* (get-num-from-user 0 3 "condition number?")))
  ;; uncomment for deploy
  (if *actr-enabled-p*
    (setf *wmt* nil)
    (setf *wmt* (make-instance 'wm-tester :snd-player *sp* :delay 3000)))
  (setf *experiment* (make-instance 'main-control-window :day day :snum subj :cond cond)) ;; :view-position #@(50 50)))
  (cond ((train-p *experiment*)
         (setf *wmt* nil)
         (setf *sw* nil))
        (t
         (setf *sw* (make-instance 'score-windoid :view-position #@(-50 -200)))
         (unless *actr-enabled-p* (reset-probe *wmt*))))
  (run-experiment *experiment*))


(defun abort-exp ()
  (menubar-show)
  (window-close (task-wind *experiment*))
  (window-close *experiment*)
  (when *sw* (window-close *sw*))
  (setf *sw* nil))



;;; Miscellaneous Utility Functions

(defun trials-left ()
  "Returns the number of trials left in the experiment. Handy for when a session runs late and the subjects wants to know how much longer the experiment will take. Only works for day 2, of course."
  (- (size (car (block-lst *experiment*))) 
     (trials-completed *experiment*)))

(defun tblk-kind (n)
  "Returns the kind of the specified block, n, of the experiment."
  (kind (nth n (block-lst *experiment*))))

(defun see-tblk-kinds (&key (lst (block-lst *experiment*)))
  "Returns a list of the kind of every block in the experiment."
  (cond 
   ((null (car lst)) nil)
   (t (cons (kind (car lst)) 
            (see-tblk-kinds :lst (cdr lst))))))

(defun set-tblk (n k)
  "Sets the kind, k, of block n."
  (setf (kind (nth n (block-lst *experiment*))) k))

;; gets the list of trials for an experiment block
;; (trial-lst (car (block-lst *experiment*)))

;; gets the kind of the first trial of the experiment block
;; (trial-kind (car (trial-lst (car (block-lst *experiment*)))))

(defun see-next-trial-kind ()
  (trial-kind (nth (trials-completed *experiment*)
                         (trial-lst (car (block-lst *experiment*))))))

(defun set-trial-kind (n k)
  "Sets the kind, k, of the nth trial."
  (setf (trial-kind (nth n (trial-lst (car (block-lst *experiment*))))) k))

(defun set-next-trial (k)
  "Sets the kind, k, of the next trial and the button for it."
  (setf (trial-kind (nth (trials-completed *experiment*) (trial-lst (car (block-lst *experiment*))))) k)
  (setf (current-btn *experiment*) (cond
                                    ((= k 0) :phaser)
                                    ((= k 1) :transporter)
                                    ((= k 2) :navigation)
                                    ((= k 3) :jammer))))

(defun set-all-trial-kinds (k &key (lst (trial-lst (car (block-lst *experiment*)))))
  "Sets all trials to kind k."
  (cond
   ((null lst) nil)
   (t (cond
       ((not (= k (trial-kind (car lst))))
        (cons (setf (trial-kind (car lst)) k) 
              (set-all-trial-kinds k :lst (cdr lst))))
       (t (cons (car lst) (set-all-trial-kinds k :lst (cdr lst))))))))


(defun see-all-trial-kinds (&key (k 0) (lst (trial-lst (car (block-lst *experiment*)))))
  (cond
   ((null lst) nil)
   (t (cons (trial-kind (nth k lst)) (see-all-trial-kinds :lst (cdr lst))))))

;; remove all navigation trials
(defun remove-nav-trials (&key (lst (trial-lst (car (block-lst *experiment*)))))
        (cond
         ((null lst) nil)
         (t (cond
             ((= 2 (trial-kind (car lst)))
              (cons (setf (trial-kind (car lst)) (car (randomize-list '(0 1 3)))) 
                    (remove-nav-trials :lst (cdr lst))))
             (t (cons (car lst) (remove-nav-trials :lst (cdr lst))))))))

(defun get-ready-to-switch ()
  (setf (pswitch-count *experiment*) 6)
  (setf (tswitch-count *experiment*) 5)
  (setf (jswitch-count *experiment*) 5))

(defun trial-list ()
  (trial-lst (car (block-lst *experiment*))))

  


;; for model development
;; (setf *sw* (make-instance 'score-windoid :view-position #@(1600 900)))


;;;; ---------------------------------------------------------------------- ;;;;
;;;; test code
#|

;;;; Execute first

(setf *sw* (make-instance 'score-windoid :view-position #@(1600 900)))
(setf *sp* 
        (make-instance 
          'snd-player :res-file nil
          :preload '("Warning" "Whit" "Phaser" "Torpedo""Whip" "damn" "expolsion"
                     "Transporter TNG" "Probe" "a" "b" "c" "d" "e" "f" "g" "h" 
                     "i" "j" "k" "l" "m" "n" "o" "p" "q" "r" "s" "t" "u" "v" 
                     "w" "x" "y" "z" "uh oh" "glass")))

;; Execute these to get the WM tester, for day 2 
(setf *wmt* (make-instance 'wm-tester :snd-player *sp* :delay 3000))
(setf *wmt* nil)


;;;; Then, to get a  window:

(setf *experiment* (make-instance 'phaser-window-no-interruption :snd-player *sp*))
(setf *experiment* (make-instance 'transporter-window-extra-buttons :snd-player *sp*))
(setf *experiment* (make-instance 'jammer-window-lock-signal :snd-player *sp*))
(setf *experiment* (make-instance 'jammer-window-scanner-off :snd-player *sp*))
(setf *experiment* (make-instance 'pump-window :snd-player *sp*))

|#

