;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Mike Byrne
;;; Copyright   : (c)2006 Rice U./Mike Byrne, All Rights Reserved
;;; Availability: public domain
;;; Address     : Rice University
;;;             : Psychology Department
;;;             : Houston,TX 77251-1892
;;;             : byrne@acm.org
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : devices.lisp
;;; Version     : r3
;;; 
;;; Description : Code for the phaser device for Frank's dissertation ACT-R 
;;;		model  
;;; 
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;; 2008.10.05 fpt [r3]
;;;	: Modified devide-update methods for phaser & transporter so that they
;;;	trigger a reward of -1 as time passes, approximately once per second.
;;;
;;; 2008.08.21 fpt [r2]
;;;	: Renamed to "devices r2.lisp" to reflect the fact that this
;;;	file will now contain all the Star Trek ACT-R devices.
;;;	: Reworked test & run functions to work with other tasks
;;;	besides just the phaser.
;;;
;;; 2008.05.18 fpt [r1]
;;;	: Folded previous attempt to get ONR model working in ACT-R 6
;;;	(r603) into Frank's dissertation project.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;






;;; Utility functions for making a task window and running trials and whatnot.

(defun test-task (kind &key (out-path t) (real-time nil))
  "Create an experiment task window."
  (unless *sp*
    (setf *sp* 
          (make-instance 
            'snd-player :res-file nil
            :preload '("Warning" "Whit" "Phaser" "Whip" "damn" "expolsion"
                       "Transporter TNG" "Probe" "a" "b" "c" "d" "e" "f" "g" "h" 
                       "i" "j" "k" "l" "m" "n" "o" "p" "q" "r" "s" "t" "u" "v" 
                       "w" "x" "y" "z" "uh oh" "glass"))))
;;  (when (and *sp* *actr-enabled-p*) (setf *sp* nil))
  (when *experiment* (window-close *experiment*))
  (unless *sw* (setf *sw* (make-instance 'score-windoid :view-position #@(1600 -900))))
  (case kind
    (0 (setf *experiment* (make-instance 'phaser-window-semantic-control 
                       :snd-player *sp* 
                       ;;                       :out-path out-path
                       :view-position #@(20 50))))
    (1 (setf *experiment* (make-instance 'transporter-window-static-subtask-order 
                       :snd-player *sp* 
                       ;;                       :out-path out-path
                       :view-position #@(20 50))))
    (3 (setf *experiment* (make-instance 'jammer-window-static-subtask-order 
                       :snd-player *sp* 
                       ;;                       :out-path out-path
                       :view-position #@(20 50))))
    (5 (setf *experiment* (make-instance 'jammer-window-0 
                       :snd-player *sp* 
                       :out-path out-path
                       :view-position #@(20 50)))))
  (when *actr-enabled-p*
    (case kind
      (0 (goal-focus phaser-goal))
      (1 (goal-focus transporter-goal))
      (3 (goal-focus jammer-goal))
      (5 (goal-focus jammer-goal)))
    (case kind
      (0 (set-buffer-chunk 'imaginal 'phaser-imaginal))
      (1 (set-buffer-chunk 'imaginal 'transporter-imaginal))
      (3 (set-buffer-chunk 'imaginal 'jammer-imaginal))
      )
    (when (= kind 5)
      (add-dm
       (changed-jammer-imaginal isa problem-state interface transporter top do-changed-jammer middle scan bottom active-scan)
       
       (intervene-jammer-scanner-on isa change-instruction current-flag scanner-on do-step active-scan do-subtask scan next-flag lock-signal)
       (intervene-jammer-lock-signal isa change-instruction current-flag lock-signal do-step scanner-off do-subtask scan next-flag enter-frequency)
       (intervene-jammer-enter-frequency isa change-instruction current-flag enter-frequency do-step lock-signal do-subtask scan next-flag scanner-off)
       (intervene-jammer-scanner-off isa change-instruction current-flag scanner-off do-step scanner-on do-subtask scan next-flag active-scan)
       (intervene-jammer-active-scan isa change-instruction current-flag active-scan do-step enter-frequency do-subtask frequency next-flag scanner-on)
       )

      (mod-chunk jammer-imaginal top do-changed-jammer)
      (mod-chunk jammer-goal state-micro scanner-on)
      (set-buffer-chunk 'imaginal 'changed-jammer-imaginal))
    (setf *reward-time* 0)
    (install-device *experiment*)
    (proc-display :clear t)
    ))

(defun run-a-trial (kind &key (time 180) (out-path t) (real-time nil))
  "Run ACT for one trial for <time> time with the given task, optionally specifying an output path."
  (reset)
  (test-task kind :out-path out-path)
  (run time :real-time real-time))
;;  (run-until-condition 'condition-error :real-time real-time))

(defun step-a-trial (kind &key (out-path t))
  "Run ACT for one trial with the given task, optionally specifying an output path."
  (reset)
  (test-task kind out-path)
  (run-step)
  )


(defun run-n-trials (n kind cond &key (switch nil) (time 180) (subj 0))
  "Run ACT for <n> trials on the given task, each one for <time> time."
  (reset)
  (begin-experiment :cond cond :subj subj)
  (window-close (task-wind *experiment*))
  (setf (single-kind *experiment*) t)
  (case kind
    (0 (setf (block-lst *experiment*) (do-stimgen (train-p *experiment*) n 0 0 0 :single-kind t)))
    (1 (setf (block-lst *experiment*) (do-stimgen (train-p *experiment*) 0 n 0 0 :single-kind t)))
    (3 (setf (block-lst *experiment*) (do-stimgen (train-p *experiment*) 0 0 0 n :single-kind t))))
  (set-next-trial kind)
  (task-window *experiment*)
  (when switch
    (case kind
      (0 (setf (pswitch *experiment*) t))
      (3 (setf (jswitch *experiment*) t))))
  (event-dispatch)
  (sleep 2)
  (run (* n time))
  )

(defun run-exp (&key (cond 3) (runs 1) (time 1200) (rt nil) (s-start 0))
  (when *experiment* (progn
                       (when (task-wind *experiment*) (window-close (task-wind *experiment*)))
                       (window-close *experiment*)
                       (setf *experiment* nil)))
  (dotimes (i runs)
    (reset)
    (setf *reward-time* 0)
    (begin-experiment :cond cond :subj (+ i (* cond runs) s-start))
    ;;  (setf *timer* (timer (task-wind *experiment*)))
;;    (goal-focus main-control-goal)
;;    (set-buffer-chunk 'imaginal 'main-control-imaginal)
;;    (install-device *experiment*)
;;    (event-dispatch)
;;    (proc-display :clear t)
    (event-dispatch)
    (sleep 2)
    (run time :real-time rt)))

(defun run-a-model (cond &key (runs 1) (time 3600) (rt nil) (s-start 0))
  (case cond
    (0 (with-model fdm0 (run-exp :cond cond :runs runs :time time :rt rt :s-start s-start)))
    (1 (with-model fdm1 (run-exp :cond cond :runs runs :time time :rt rt :s-start s-start)))
    (2 (with-model fdm2 (run-exp :cond cond :runs runs :time time :rt rt :s-start s-start)))
    (3 (with-model fdm3 (run-exp :cond cond :runs runs :time time :rt rt :s-start s-start)))))

(defun run-all-models (&key (runs 1) (time 3600) (rt nil) (s-start 0))
  (dotimes (i 4) 
    (reset)
    (event-dispatch)
    (sleep 2)
    (run-a-model i :runs runs :time time :rt rt :s-start s-start)))

  


;;;; ---------------------------------------------------------------------- ;;;;
;;;; RPM device interface
;;;; ---------------------------------------------------------------------- ;;;;



#|
;; This is probably a good simple example of a build-features-for method.
(defmethod build-features-for ((self polygon-view) (vm vision-module))
  (make-instance 'icon-feature
    :screen-obj self :color (system-color->symbol (color self))
    :x (px (view-loc self)) :y (py (view-loc self))
    :value 'triangle))

(defmethod device-handle-click ((device phaser-window))
  (view-click-event-handler device (view-mouse-position device))
  (event-dispatch))
|#


(setf *actr-enabled-p* t)



(defmethod advance-state :after ((wind procedure-window))
  (when *actr-enabled-p* (proc-display))
;;  (format t "~%advance-state")
  )




;;; Phaser


(defmethod device-update ((wind phaser-window) time)
  (when (<= 0.050 (- time (last-update wind)))
    (let (dirty?)
      (setf (last-update wind) time)
      (when (charge-p wind)
        (let* ((thermo (view-named :thermo wind)))
          (unless (> (thermometer-value thermo) 0)
            (setf (thermometer-value thermo) 70))
          )
        (setf dirty? t))
      (when (track-p wind)
        (set-view-position (trgt-v wind) (trgt-cntr wind))
;        (move-target wind (idle-move wind))
        (setf dirty? t))
      (when (not (equal "" (dialog-item-text (view-named :status wind))))
        (setf dirty? t))
      
#|
      (when (and (not (training-p wind))
                 (not (= (elapsed-secs wind)
                         (read-from-string 
                          (dialog-item-text (view-named :TIME wind))))))
        (set-dialog-item-text (view-named :TIME wind)
                              (mkstr (elapsed-secs wind)))
        (setf dirty? t))
|#
      (when dirty? (proc-display))
    (when (>= (- time *reward-time*) *time-elapse-reward-interval*)
      (trigger-reward *time-elapse-reward*)
      (incf *reward-time* *time-elapse-reward-interval*))
  )))




(defmethod build-features-for ((self target-sv) (vm vision-module))
  (make-instance 'icon-feature
    :x (px (view-loc self))
    :y (py (view-loc self))
    :screen-obj self
    :value 'blip
    ))


(defmethod build-features-for ((self thermometer) (vm vision-module))
  (let* ((val (thermometer-value self))
         (pos (view-position self))
         (dim (view-size self))
         (ypos (+ (point-v pos) (round (* (point-v dim)
                                           (- 1 (/ val 100))))))
         )
    ;; only generate a feature this when the value is greater than zero
    (when (> val 0)
      (list
       (make-instance 'line-feature
         :x (px (view-loc self))
         :y ypos
         :end1-x (point-h pos)
         :end2-x (+ (point-h pos) (point-h dim))
         :end1-y ypos
         :end2-y ypos
         :screen-obj self)
       ))))


(defmethod build-features-for ((self click-indicator) (vm vision-module))
  (let*  ((vsize (view-size self))
          (tmp 
           (make-instance 'rect-feature
             :screen-obj self :color 'BLACK
             :x (px (view-loc self))
             :y (+ (point-v (view-position self)) (round (point-v vsize) 4))
             :width (point-v vsize)
             :height (round (point-h vsize) 2)
             )))
    (aif (indicator self)
      (cons tmp
            (make-instance 'icon-feature
              :x (+ (point-h (view-position self)) it)
              :y (+ (point-v (view-position self)) (v-offset self) 10)
              :screen-obj self
              :height 18
              :width 34
              :value 'triangle ))
      tmp)))


    


;;; Transporter

(defmethod device-update ((wind transporter-window) time)
  (when (<= 0.050 (- time (last-update wind)))
    (let (dirty?)
      (setf (last-update wind) time)
      (when (scan-p wind) (check-scan wind))
;;      (when (scan-p wind)  (check-scan-act wind))
      (when (not (equal "" (dialog-item-text (view-named :status wind))))
        (setf dirty? t))



#|
      (when (and (not (training-p wind))
                 (not (= (elapsed-secs wind)
                         (read-from-string 
                          (dialog-item-text (view-named :TIME wind))))))
        (set-dialog-item-text (view-named :TIME wind)
                              (mkstr (elapsed-secs wind)))
        (setf dirty? t))
|#
      (when dirty? (proc-display))
    (when (>= (- time *reward-time*) *time-elapse-reward-interval*)
      (trigger-reward *time-elapse-reward*)
      (incf *reward-time* *time-elapse-reward-interval*))
  )))

#|
(defmethod device-update ((wind main-control-window) time)
  nil)

  (when (>= (- time *reward-time*) 1)
    (trigger-reward *time-elapse-reward*)
    (incf *reward-time*)))
|#

#| depricated
(defun model-wait (wait-time)
  (let ((start (mp-time)))
        (while (> wait-time (- (mp-time) start)))))
|#

;;; Main Control
#|
(defmethod advance-state :after ((wind main-control-window))
  (when *actr-enabled-p* (proc-display)))
|#

#|
(defmethod check-scan-act ((wind transporter-window))
  (when (and (< (scan-rate wind)
                (- (pm-get-time) (last-scan wind))))
    (setf (last-scan wind) (pm-get-time))
    (when (and (= (random 3) 1)
               (> (scan-count wind) 1))
      (remove-subviews wind (scan-marker wind (scan-count wind)))
      (decf (scan-count wind)))
    (let ((width (second (assoc (scan-count wind)
                                 '((1 36) (2 50) (3 82) (4 144))))))
      (dotimes (i (scan-count wind))
        (set-view-position (scan-marker wind (1+ i))
                           (+ 641 (- (/ width 2)) (random width))
                           (+ 91 (- (/ width 2)) (random width)))))))
|#




(defun show-me-all-the-buffers ()
  (buffer-chunk goal)
  (buffer-chunk imaginal)
  (buffer-chunk retrieval)
  (buffer-chunk visual-location)
  (buffer-chunk visual)
  (buffer-chunk aural-location)
  (buffer-chunk manual))